@@ -1,4 +1,4 @@
-const char rcsid_kd_add_c[] = "$Id: kd_add.c,v 1.41 1999/05/31 20:21:07 marc Exp $";
+const char rcsid_kd_add_c[] = "$Id: kd_add.c,v 1.7 2003/01/25 16:32:02 dmshaw Exp $";
 
 /* 
  * Copyright (c) 1996, 1997, 1998, 1999, Marc Horowitz.  All rights reserved.
@@ -56,6 +56,11 @@ typedef struct _dkm_state {
    error *err;
 } dkm_state;
 
+typedef struct _dkm1_state {
+   kd_txn tid;
+   dkm_state *dkms;
+} dkm1_state;
+
 int words_elem_merge(llist *lout, void *e1, void *e2, void *c)
 {
    int o;
@@ -613,12 +618,14 @@ int kd_keys_elem_merge(llist *lout, void *e1, void *e2, void *c)
       if (s)
 	 s->not_changed_revocation_sig++;
       /*
-       * Go on with the keyring, but keep the old revocation
-       * (free the new one to avoid memory leaks)
+       * Go on with the keyring, but delete the new revocation
+       * so it will not be accounted for below (avoids incremental loops)
        */
       xbuffer_free(&(ke2->revocation));
+/*
       ke2->revocation = ke1->revocation;
       xbuffer_alloc(&(ke1->revocation));
+*/
    }
 
    if (s)
@@ -746,7 +753,7 @@ int kd_keys_elem_merge(llist *lout, void *e1, void *e2, void *c)
 
 	  ((ke->subkey = ke2->subkey),
 	   (ke->subkeysig = ke2->subkeysig),
-
+	   (ke->disabled= 0),
 	   !llist_add(&(s->add_keys), ke))) {
 	 dabort();
 	 return(LLIST_MERGE_FAIL);
@@ -784,6 +791,7 @@ int add_keys_elem_free(void *e, void *c)
 }
 
 typedef struct _aetd_state {
+   kd_txn tid;
    unsigned char entry[12];
    int writing;
    int pks_errno;
@@ -798,7 +806,7 @@ int add_entry_to_db(DB *db, DBT *key, aetd_state *s)
    memset(&data, 0, sizeof(data));
    memset(&newdata, 0, sizeof(newdata));
 
-   ret = (*(db->get))(db, NULL, key, &data, 0);
+   ret = (*(db->get))(db, s->tid, key, &data, 0);
 
    if (ret == DB_NOTFOUND) {
       /* word not in file, create DBT for insert */
@@ -870,7 +878,7 @@ int add_entry_to_db(DB *db, DBT *key, aetd_state *s)
       }
    }
 
-   if ((ret = (*(db->put))(db, NULL, key, &newdata, 0))) {
+   if ((ret = (*(db->put))(db, s->tid, key, &newdata, 0))) {
       s->writing = 1;
       s->pks_errno = ret;
       if (needfree)
@@ -898,7 +906,7 @@ int add_word_to_worddb(void *e, void *c)
    memset(&key, 0, sizeof(key));
    memset(&data, 0, sizeof(data));
 
-   for (i=0; i<we->len; i++)
+   for (i=0; i<we->len && i < sizeof(word); i++)
       word[i] = tolower((we->ptr)[i]);
 
    key.data = (void *) word;
@@ -907,7 +915,7 @@ int add_word_to_worddb(void *e, void *c)
    data.data = s->entry;
    data.size = 12;
 
-   if ((ret = (*(worddb->put))(worddb, NULL, &key, &data, 0))) {
+   if ((ret = (*(worddb->put))(worddb, s->tid, &key, &data, 0))) {
       s->writing = 1;
       s->pks_errno = ret;
       fail();
@@ -929,10 +937,11 @@ int add_key_to_worddb(void *e, void *c)
    return(1);
 }
 
-int db_store_wordlist(llist *keys, error *err)
+int db_store_wordlist(kd_txn tid, llist *keys, error *err)
 {
    aetd_state aetds;
 
+   aetds.tid = tid;
    aetds.pks_errno = 0;
 
    if (!llist_iterate(keys, add_key_to_worddb, &aetds)) {
@@ -978,10 +987,11 @@ int add_key_to_timedb(void *e, void *c)
    return(add_entry_to_db(timedb, &key, s));
 }
 
-int db_store_timestamp(llist *keys, error *err)
+int db_store_timestamp(kd_txn tid, llist *keys, error *err)
 {
    aetd_state aetds;
 
+   aetds.tid = tid;
    aetds.pks_errno = 0;
 
    /* all the keyid's are the same, only need to deal with the first,
@@ -1008,10 +1018,10 @@ const char bad_key_errstr[] =
 "unprocessable key is %02X%02X%02X%02X.\n";
 const char revocation_sig[] = "revocation signature";
 
-int db_key_merge(void *e, void *c)
+int db_key_merge_1(void *e, void *c)
 {
    keys_elem *ke = (keys_elem *) e;
-   dkm_state *s = (dkm_state *) c;
+   dkm1_state *s = (dkm1_state *) c;
 
    llist db_keys;
 
@@ -1019,15 +1029,15 @@ int db_key_merge(void *e, void *c)
 
    /* first, clear the disabled bit in files from a user */
 
-   if (s->no_strip_disabled && ke->disabled)
+   if (s->dkms->no_strip_disabled && ke->disabled)
       ke->disabled = -1;
    else
       ke->disabled = 0;
 
-   ke->disabled = s->no_strip_disabled?(ke->disabled?-1:0):0;
+   ke->disabled = s->dkms->no_strip_disabled?(ke->disabled?-1:0):0;
 
-   if (!kd_get_keys_by_keyid(ke->keyidbits.buf /* 8 byte keyid */,
-			     &db_keys, s->err))
+   if (!kd_get_keys_by_keyid(s->tid, ke->keyidbits.buf /* 8 byte keyid */,
+			     &db_keys, s->dkms->err))
       return(0);
 
    if (llist_count(&db_keys) == 0) {
@@ -1037,22 +1047,22 @@ int db_key_merge(void *e, void *c)
       if (!llist_add(&db_keys, e)) {
 	 llist_free(&db_keys);
 
-	 s->err->fatal = 1;
-	 s->err->str = "adding request key to db_keys failed";
+	 s->dkms->err->fatal = 1;
+	 s->dkms->err->str = "adding request key to db_keys failed";
 	 fail();
       }
 
-      s->ms.new_pubkeys++;
-      if (s->ms.verbose)
-	 display_new_key(&(s->ms), &(ke->keyidbits));
+      s->dkms->ms.new_pubkeys++;
+      if (s->dkms->ms.verbose)
+	 display_new_key(&(s->dkms->ms), &(ke->keyidbits));
 
       /* this is definitely a new key.  marshall it into the add
 	 xbuffer for later */
 
-      if (s->ms.save_add) {
-	 if (!kd_keys_elem_marshall(ke, s->ms.add_xb)) {
-	    s->err->fatal = 1;
-	    s->err->str = "failed marshalling new key";
+      if (s->dkms->ms.save_add) {
+	 if (!kd_keys_elem_marshall(ke, s->dkms->ms.add_xb)) {
+	    s->dkms->err->fatal = 1;
+	    s->dkms->err->str = "failed marshalling new key";
 	    fail();
 	 }
       }
@@ -1071,8 +1081,8 @@ int db_key_merge(void *e, void *c)
 	 llist_iterate(&db_keys, keys_elem_free, NULL);
 	 llist_free(&db_keys);
 
-	 s->err->fatal = 1;
-	 s->err->str = "adding request key to request_key failed";
+	 s->dkms->err->fatal = 1;
+	 s->dkms->err->str = "adding request key to request_key failed";
 	 fail();
       }
 
@@ -1081,7 +1091,7 @@ int db_key_merge(void *e, void *c)
       llist_alloc(&merged_keys);
    
       if (!llist_merge(&merged_keys, &db_keys, &request_key,
-		       kd_keys_elem_merge, (void *) &(s->ms),
+		       kd_keys_elem_merge, (void *) &(s->dkms->ms),
 		       keys_elem_free, NULL)) {
 	 /* if this ever becomes non-fatal, remember this:
 	    // merge will destroy the input lists
@@ -1091,38 +1101,39 @@ int db_key_merge(void *e, void *c)
 	    keys_elem_alloc(ke);
 	    */
 
-	 s->err->fatal = 1;
-	 s->err->str = "merging database key with request key failed";
+	 s->dkms->err->fatal = 1;
+	 s->dkms->err->str = "merging database key with request key failed";
 	 fail();
       }
 
       db_keys = merged_keys;
 
-      if (s->ms.save_add) {
+      if (s->dkms->ms.save_add) {
 	 /* check to see if anything was added, and if so, marshall it
 	    into the add xbuffer for later */
 
-	 if (!llist_iterate(&(s->ms.add_keys), kd_keys_elem_marshall,
-			    s->ms.add_xb) ||
-	     !llist_iterate(&(s->ms.add_keys), add_keys_elem_free, NULL)) {
-	    s->err->fatal = 1;
-	    s->err->str = "failed marshalling added keys";
+	 if (!llist_iterate(&(s->dkms->ms.add_keys), kd_keys_elem_marshall,
+			    s->dkms->ms.add_xb) ||
+	     !llist_iterate(&(s->dkms->ms.add_keys), add_keys_elem_free,
+			    NULL)) {
+	    s->dkms->err->fatal = 1;
+	    s->dkms->err->str = "failed marshalling added keys";
 	    fail();
 	 }
 
-	 llist_free(&(s->ms.add_keys));
+	 llist_free(&(s->dkms->ms.add_keys));
       }
    }
 
    /* check to see if the key actually changed.  Don't bother writing
       if nothing did */
 
-   if (!s->ms.new_sigs &&
-       !s->ms.repl_sigs &&
-       !s->ms.new_userids &&
-       !s->ms.changed_primary_userids &&
-       !s->ms.new_revocations &&
-       !s->ms.new_pubkeys) {
+   if (!s->dkms->ms.new_sigs &&
+       !s->dkms->ms.repl_sigs &&
+       !s->dkms->ms.new_userids &&
+       !s->dkms->ms.changed_primary_userids &&
+       !s->dkms->ms.new_revocations &&
+       !s->dkms->ms.new_pubkeys) {
       llist_iterate(&db_keys, keys_elem_free, NULL);
       llist_free(&db_keys);
 
@@ -1140,21 +1151,21 @@ int db_key_merge(void *e, void *c)
 
    /* any errors past this point are fatal */
 
-   if (!kd_db_store_keyblock(&db_keys, s->err)) {
+   if (!kd_db_store_keyblock(s->tid, &db_keys, s->dkms->err)) {
       llist_iterate(&db_keys, keys_elem_free, NULL);
       llist_free(&db_keys);
 
       fail();
    }
 
-   if (!db_store_wordlist(&db_keys, s->err)) {
+   if (!db_store_wordlist(s->tid, &db_keys, s->dkms->err)) {
       llist_iterate(&db_keys, keys_elem_free, NULL);
       llist_free(&db_keys);
 
       fail();
    }
 
-   if (!db_store_timestamp(&db_keys, s->err)) {
+   if (!db_store_timestamp(s->tid, &db_keys, s->dkms->err)) {
       llist_iterate(&db_keys, keys_elem_free, NULL);
       llist_free(&db_keys);
 
@@ -1167,6 +1178,23 @@ int db_key_merge(void *e, void *c)
    return(1);
 }
 
+int db_key_merge(void *e, void *c)
+{
+   dkm1_state dkm1s;
+   dkm_state *s = (dkm_state *) c;
+
+   dkm1s.dkms = s;
+
+   if (kd_txn_begin(&(dkm1s.tid), s->err) &&
+       db_key_merge_1(e, &dkm1s) &&
+       kd_txn_commit(dkm1s.tid, s->err))
+      return(1);
+
+   kd_txn_abort(dkm1s.tid, s->err);
+
+   return(0);
+}
+
 int kd_add_1(unsigned char *keys, long len, int flags,
 	     xbuffer *win_msg, xbuffer *newkeys_xb, error *err)
 {
@@ -1207,7 +1235,7 @@ int kd_add_1(unsigned char *keys, long len, int flags,
       database, merge the two, and write out records to all three
       databases. */
 
-   ret = kd_keyblock_iterate(keys, len, db_key_merge, &s, err, &softerr);
+   ret = kd_keyblock_iterate(keys, len, db_key_merge, &s, err, &softerr, 1);
 
    kd_sync();
 
@@ -1435,7 +1463,6 @@ int kd_add(unsigned char *keys, long len, int flags,
 {
    error err;
    xbuffer msg, addkeys;
-   kd_txn tid;
 
    err.str = err.buf;
    xbuffer_alloc(&msg);
@@ -1445,9 +1472,7 @@ int kd_add(unsigned char *keys, long len, int flags,
 
    kd_log_start("kd_add", NULL, 0, flags);
 
-   if (kd_txn_begin(&tid, &err) &&
-       kd_add_1(keys, len, flags, &msg, newkeys?&addkeys:NULL, &err) &&
-       kd_txn_commit(tid, &err)) {
+   if (kd_add_1(keys, len, flags, &msg, newkeys?&addkeys:NULL, &err)) {
       *ret = msg.buf;
       *retlen = msg.len;
       if (newkeys) {
@@ -1460,8 +1485,6 @@ int kd_add(unsigned char *keys, long len, int flags,
       return(1);
    }
 
-   kd_txn_abort(tid, NULL);
-
    if (!err.fatal) {
       if (!(*ret = (unsigned char *) my_strdup(err.str))) {
 	 err.fatal = 1;
