@@ -1,4 +1,4 @@
-const char rcsid_kd_generic_c[] = "$Id: kd_generic.c,v 1.19 1999/05/31 20:21:11 marc Exp $";
+const char rcsid_kd_generic_c[] = "$Id: kd_generic.c,v 1.4 2002/09/08 20:54:24 rlaager Exp $";
 
 /* 
  * Copyright (c) 1996, 1997, 1998, 1999, Marc Horowitz.  All rights reserved.
@@ -149,7 +149,7 @@ int kd_keys_elem_marshall(void *e, void *c)
   return 1;
 }
 
-int kd_db_store_keyblock(llist *keys, error *err)
+int kd_db_store_keyblock(kd_txn tid, llist *keys, error *err)
 {
    DBT key, newdata;
    xbuffer newxb;
@@ -176,7 +176,7 @@ int kd_db_store_keyblock(llist *keys, error *err)
    newdata.data = (void *) newxb.buf;
    newdata.size = (size_t) newxb.len;
 
-   if ((*(keydb(&key)->put))(keydb(&key), NULL, &key, &newdata, 0) < 0) {
+   if ((*(keydb(&key)->put))(keydb(&key), tid, &key, &newdata, 0) < 0) {
       xbuffer_free(&newxb);
       err->fatal = 1;
       sprintf(err->buf, "error %s keydb, errno = %d", "writing to", errno);
@@ -235,6 +235,7 @@ int kd_txn_begin(kd_txn *tid, error *err)
 int kd_txn_commit(kd_txn tid, error *err)
 {
     int ret;
+    static int txn_count = 0;
 
     if (! tid)
 	return(1);
@@ -244,9 +245,24 @@ int kd_txn_commit(kd_txn tid, error *err)
 	err->str = "Failed committing transaction";
 	return(0);
     }
-
-    return(1);
-}
+ 
+     txn_count++;
+     if (txn_count > KD_MAX_TXN_WO_CKPT) {
+       int count;
+       for (count=0; count<10; count++) {
+          if ((ret = txn_checkpoint(dbenv.tx_info, 0, 0)) == DB_INCOMPLETE) {
+             sleep(1);
+             continue;
+          }
+         /* Only reset if successful */
+         txn_count = 0;
+         break;
+       }
+      }
+  
+      return(1);
+  }
+  
 
 int kd_txn_abort(kd_txn tid, error *err)
 {
@@ -665,7 +681,9 @@ void kd_sync_1()
 
 void kd_close_1()
 {
-   int i, ret;
+   int count;
+   int i;
+   int ret = 0;
    char buf[MAXPATHLEN+100];
    char **loglist, **logfileptr;
 
@@ -678,38 +696,58 @@ void kd_close_1()
 	 (*(keydb_files[i]->close))(keydb_files[i], 0);
 
    if (dbenv.tx_info) {
-       /* I don't know why I need to do this twice, but if I don't,
-	  log_archive() returns no files */
+      /* I don't know why I need to do this twice, but if I don't,
+	 log_archive() returns no files */
 
-       if ((ret = txn_checkpoint(dbenv.tx_info, 0, 0))) {
-	   sprintf(buf, "failed db checkpointing: errno = %d", ret);
-	   log_error("kd_close_1", buf);
-	   return;
-       }
-       if ((ret = txn_checkpoint(dbenv.tx_info, 0, 0))) {
-	   sprintf(buf, "failed db checkpointing: errno = %d", ret);
-	   log_error("kd_close_1", buf);
-	   return;
-       }
+      for (count=0; count<10; count++) {
+	 if ((ret = txn_checkpoint(dbenv.tx_info, 0, 0)) == DB_INCOMPLETE) {
+	    sleep(1);
+	    continue;
+	 }
 
-       if ((ret = log_archive(dbenv.lg_info, &loglist, DB_ARCH_ABS, NULL))) {
-	   sprintf(buf, "failed listing unneeded log files: errno = %d", ret);
-	   log_error("kd_close_1", buf);
-	   return;
-       }
+	 break;
+      }
 
-       if (loglist) {
-	   for (logfileptr = loglist; *logfileptr; logfileptr++) {
-	       if ((ret = unlink(*logfileptr))) {
-		   sprintf(buf, "failed removing log file %s: errno = %d",
-			   *logfileptr, ret);
-		   log_error("kd_close_1", buf);
-		   return;
-	       }
-	   }
+      if (ret) {
+	 sprintf(buf, "failed db checkpointing: errno = %d", ret);
+	 log_error("kd_close_1", buf);
+	 return;
+      }
 
-	   free(loglist);
-       }
+      for (count=0; count<10; count++) {
+	 if ((ret = txn_checkpoint(dbenv.tx_info, 0, 0)) == DB_INCOMPLETE) {
+	    sleep(1);
+	    continue;
+	 }
+
+	 break;
+      }
+
+      if (ret) {
+	 sprintf(buf, "failed db checkpointing: errno = %d", ret);
+	 log_error("kd_close_1", buf);
+	 return;
+      }
+
+      if ((ret = log_archive(dbenv.lg_info, &loglist, DB_ARCH_ABS, NULL))) {
+	 sprintf(buf, "failed listing unneeded log files: errno = %d", ret);
+	 log_error("kd_close_1", buf);
+	 return;
+      }
+
+
+      if (loglist) {
+	 for (logfileptr = loglist; *logfileptr; logfileptr++) {
+	    if ((ret = unlink(*logfileptr))) {
+	       sprintf(buf, "failed removing log file %s: errno = %d",
+		       *logfileptr, ret);
+	       log_error("kd_close_1", buf);
+	       return;
+	    }
+	 }
+
+	 free(loglist);
+      }
    }
 
    if ((ret = db_appexit(&dbenv))) {
