@@ -1,4 +1,4 @@
-const char rcsid_kd_search_c[] = "$Id: kd_search.c,v 1.41 1999/05/31 20:21:15 marc Exp $";
+const char rcsid_kd_search_c[] = "$Id: kd_search.c,v 1.11 2003/02/02 17:22:27 dmshaw Exp $";
 
 /* 
  * Copyright (c) 1996, 1997, 1998, 1999, Marc Horowitz.  All rights reserved.
@@ -36,6 +36,10 @@ typedef struct _mke_state {
    /* error returns */
    error *err;
    ki_softerr *softerr;
+   /* used to not show errors in certain cases, so that
+      already-mangled keys in the database don't cause errors when
+      searched for */
+   int errorhack;
 } mke_state;
 
 int finish_userid(mke_state *s)
@@ -246,6 +250,8 @@ int make_keys_elem(ddesc *packet, void *c)
       {
 	 mpidesc m, e;
 	 unsigned char keyid[8];
+	 unsigned char keytype;
+	 time_t create_time;
 
 	 s->ignoring = 0;
 
@@ -270,7 +276,8 @@ int make_keys_elem(ddesc *packet, void *c)
 	 /* extract the creation time and modulus bits from the pubkey */
 
 	 if (!decode_pubkey(packet, plen, &m, &e, keyid,
-			    &(s->ke->keytype), &(s->ke->create_time))) {
+			    &keytype, &create_time,
+			    &(s->ke->keyversion))) {
             soft_err(s, "Key block corrupt: pubkey decode failed");
             s->ignoring = 1; /* Ignore until next pubkey */
 	    return(1);
@@ -282,9 +289,17 @@ int make_keys_elem(ddesc *packet, void *c)
 	       fail();
 	 }
 
+#ifdef DBKEY_BUG
+	 s->ke->create_time=create_time;
+#endif
+
 	 /* store the modulus bits, exponent bits, and pubkey packet */
 
 	 if (ptype == 6) {			/* public key packet */
+#ifndef DBKEY_BUG
+	    s->ke->create_time=create_time;
+#endif
+	    s->ke->keytype=keytype;
 	    s->ke->modsigbits = m.nbits;
 
 	    if (!xbuffer_append(&(s->ke->pubkey), packet->data,
@@ -296,7 +311,7 @@ int make_keys_elem(ddesc *packet, void *c)
 
 	    /* for RSA keys, stash the data needed to compute the
 	       fingerprint */
-	    if (s->ke->keytype == 1) {		/* RSA */
+	    if (keytype == 1) {		/* RSA */
 	       if (!xbuffer_append(&(s->ke->modbits),
 				   m.number.data, m.number.size)) {
 		  s->err->fatal = 1;
@@ -312,6 +327,12 @@ int make_keys_elem(ddesc *packet, void *c)
 	       }
 	    }
 	 } else {				/* subkey packet */
+	    if (s->ke->subkey.len) {
+	      if(s->errorhack)
+                soft_err(s,"Key block corrupt: more than one subkey");
+	      return(1);
+	    }
+
 	    if (!xbuffer_append(&(s->ke->subkey), packet->data, packet->size)) {
 	       s->err->fatal = 1;
 	       s->err->str = "Appending subkey to key failed";
@@ -379,12 +400,18 @@ int make_keys_elem(ddesc *packet, void *c)
 	 long sig_time;
 	 sigs_elem *se;
 	 int ret;
-
+         static unsigned char maxid[8] = {0xff, 0xff, 0xff, 0xff,
+                                          0xff, 0xff, 0xff, 0xff};
 	 if (s->ignoring)
 	    break;
 
 	 /* extract the keyid and sigclass from the signature */
 
+         /* if no keyid is found, take maxid */
+         keyid.data = maxid;
+         keyid.size = 8;
+         keyid.offset = 0;
+
 	 if (!decode_sig(packet, plen, &keyid, &sigclass, &sig_time)) {
              soft_err(s,
                       "Key block corrupt: signature decode failed");
@@ -426,6 +453,10 @@ int make_keys_elem(ddesc *packet, void *c)
 	       s->err->str = "Storing subkey sig failed";
 	       fail();
 	    }
+	 } else if (sigclass == 0x28) {
+	   if(s->errorhack)
+	     soft_err(s,"Key block corrupt: subkey revocation not supported");
+	   return(1);
 	 } else {
 	    if (!s->ue) {
                 soft_err(s, "Key block corrupt: signature before userid");
@@ -560,7 +591,7 @@ int make_keys_elem(ddesc *packet, void *c)
    
 int kd_keyblock_iterate(unsigned char *block, long blocklen,
 			llist_iter iter, void *c, error *err,
-			ki_softerr *softerr)
+			ki_softerr *softerr, int errorhack)
 {
    ddesc keyblock;
    mke_state mkes;
@@ -572,6 +603,7 @@ int kd_keyblock_iterate(unsigned char *block, long blocklen,
    mkes.ue = NULL;
    mkes.err = err;
    mkes.softerr = softerr;
+   mkes.errorhack = errorhack;
 
    keyblock.data = block;
    keyblock.size = blocklen;
@@ -638,7 +670,8 @@ int add_key_to_keylist(void *e, void *c)
    return(1);
 }
 
-int kd_get_keys_by_keyid(unsigned char *keyid, llist *keys, error *err)
+int kd_get_keys_by_keyid(kd_txn tid, unsigned char *keyid,
+			 llist *keys, error *err)
 {
    DBT key, data;
    int ret;
@@ -650,7 +683,7 @@ int kd_get_keys_by_keyid(unsigned char *keyid, llist *keys, error *err)
    key.data = keyid+(8-KEYDB_KEYID_BYTES);
    key.size = KEYDB_KEYID_BYTES;
 
-   ret = (*(keydb(&key)->get))(keydb(&key), NULL, &key, &data, 0);
+   ret = (*(keydb(&key)->get))(keydb(&key), tid, &key, &data, 0);
 
    if ((ret == DB_NOTFOUND) ||
        ((ret == 0) && (data.size == 0))) {
@@ -672,7 +705,7 @@ int kd_get_keys_by_keyid(unsigned char *keyid, llist *keys, error *err)
 
    if (!kd_keyblock_iterate((unsigned char *) data.data,
 			    (long) data.size,
-			    add_key_to_keylist, &aktks, err, NULL)) {
+			    add_key_to_keylist, &aktks, err, NULL, 0)) {
       llist_iterate(keys, keys_elem_free, NULL);
 
       /* a "non-fatal" error here indicates a user err, which
@@ -728,6 +761,41 @@ int match_keys_elem_exact(void *e, void *c)
    return(1);
 }
 
+#ifdef KEYID_TRANSFORM
+int v4_v3_keyid_transform(void *e, void *c)
+{
+  keys_elem *ke=(keys_elem *)e;
+  unsigned char *wde = (unsigned char *) c;
+
+  /* This does some magic to work around a bug in previous version of
+     pksd.  The problem is that they calculated v4 RSA keyids as if
+     they were v3, and stored them in the database as that.  So, in
+     order to fix this, we catch cases where the actual keyid does not
+     match the keyid that the database thinks we have and substitute
+     what the database thinks.  Thus, key deletions work without
+     leaving the database in an inconsistent state.  Eventually this
+     function should be removed. - dshaw */
+
+  if(ke->keytype==1 && ke->keyversion>3
+     && (ke->keyidbits.buf[0]!=wde[0]
+	 || ke->keyidbits.buf[1]!=wde[1]
+	 || ke->keyidbits.buf[2]!=wde[2]
+	 || ke->keyidbits.buf[3]!=wde[3]
+	 || ke->keyidbits.buf[4]!=wde[4]
+	 || ke->keyidbits.buf[5]!=wde[5]
+	 || ke->keyidbits.buf[6]!=wde[6]
+	 || ke->keyidbits.buf[7]!=wde[7]))
+    {
+      /* We have a mismatch between what the keyid is and what it was
+	 looked up as. */
+
+      memcpy(ke->keyidbits.buf,wde,8);
+    }
+
+  return (1);
+}
+#endif
+
 int match_keys_elem(void *e, void *c)
 {
    keys_elem *ke = (keys_elem *) e;
@@ -822,6 +890,7 @@ int wdes_intersect(llist *lout, void *e1, void *e2, void *c)
 }
 
 typedef struct _wki_state {
+   kd_txn tid;
    int first;
    llist wdes;
    error *err;
@@ -853,16 +922,27 @@ int word_key_intersect(void *e, void *c)
    if (!s->first && (llist_count(&(s->wdes)) == 0))
       return(1);
 
+   /* a word is greater than the maximum userid size.  fail.  This shouldn't
+      ever happen, since the total userid length is checked, but it makes
+      analyzing the code for buffer overruns easier */
+
+   if (we->len > sizeof(word)) {
+       s->err->fatal = 0;
+       sprintf(s->err->buf, "search word is greater than %d characters",
+	       sizeof(word));
+       return(0);
+   }
+
    memset(&key, 0, sizeof(key));
    memset(&data, 0, sizeof(data));
 
-   for (i=0; i<we->len; i++)
+   for (i=0; i<we->len && i < sizeof(word); i++)
       word[i] = tolower((we->ptr)[i]);
 
    key.data = (void *) word;
    key.size = (size_t) we->len;
 
-   ret = (*(worddb->cursor))(worddb, NULL, &cursor, 0);
+   ret = (*(worddb->cursor))(worddb, s->tid, &cursor, 0);
 
    if (ret && ret != DB_NOTFOUND) {
       s->err->fatal = 1;
@@ -881,6 +961,7 @@ int word_key_intersect(void *e, void *c)
 	 if ((wde = (unsigned char *) malloc(12)) == NULL) {
 	    llist_iterate(&(db_wdes), malloc_elem_free, NULL);
 	    llist_free(&(db_wdes));
+	    (*(cursor->c_close))(cursor);
 	    s->err->fatal = 1;
 	    s->err->str = "allocating memory for wde failed";
 	    fail();
@@ -893,6 +974,7 @@ int word_key_intersect(void *e, void *c)
 	 if (!llist_add(&db_wdes, (void *) wde)) {
 	    llist_iterate(&(db_wdes), malloc_elem_free, NULL);
 	    llist_free(&(db_wdes));
+	    (*(cursor->c_close))(cursor);
 	    s->err->fatal = 1;
 	    s->err->str = "failed adding keyid to db_wdes list";
 	    fail();
@@ -900,6 +982,7 @@ int word_key_intersect(void *e, void *c)
       }
 
       if (ret && ret != DB_NOTFOUND) {
+	 (*(cursor->c_close))(cursor);
 	 s->err->fatal = 1;
 	 sprintf(s->err->buf, "error reading from worddb cursor: error = %d",
 		 ret);
@@ -912,6 +995,8 @@ int word_key_intersect(void *e, void *c)
       llist_free(&(s->wdes));
    }
 
+   (*(cursor->c_close))(cursor);
+
    /* the first time, just take the list as-is */
 
    if (s->first) {
@@ -947,9 +1032,17 @@ int kd_output_wde(void *e, void *c)
 
    llist_alloc(&db_keys);
 
-   if (!kd_get_keys_by_keyid(wde+4, &db_keys, s->err))
+   if (!kd_get_keys_by_keyid(s->tid, wde+4, &db_keys, s->err))
       return(0);
 
+#ifdef KEYID_TRANSFORM
+   /* If the keyid we got doesn't match the keyid we looked up, check
+      if the key is v4 rsa.  if it is, change the keyid internally to
+      the old bogus one */
+   if (!llist_iterate(&db_keys, v4_v3_keyid_transform, wde+4))
+      return(0);
+#endif
+
    create_time = ((wde[0]<<24)+(wde[1]<<16)+(wde[2]<<8)+(wde[3]));
 
    if (!llist_count(&db_keys)) {
@@ -982,7 +1075,7 @@ int kd_output_wde(void *e, void *c)
 
    if (s->filter) {
       if (llist_count(&(ow1s.new_keys_elem))) {
-	 if (!kd_db_store_keyblock(&(ow1s.new_keys_elem), s->err))
+	 if (!kd_db_store_keyblock(s->tid, &(ow1s.new_keys_elem), s->err))
 	    return(0);
 
 	 llist_iterate(&(ow1s.new_keys_elem), keys_elem_free, NULL);
@@ -995,7 +1088,7 @@ int kd_output_wde(void *e, void *c)
 	 key.data = wde+4 + (8-KEYDB_KEYID_BYTES);
 	 key.size = KEYDB_KEYID_BYTES;
 
-	 if ((*(keydb(&key)->del))(keydb(&key), NULL, &key, 0)) {
+	 if ((*(keydb(&key)->del))(keydb(&key), s->tid, &key, 0)) {
 	    s->err->fatal = 1;
 	    s->err->str = "failed deleting keydb entry from database";
 	    fail();
@@ -1048,7 +1141,7 @@ int func_then_free(void *e, void *c)
    return(1);
 }
 
-int do_all_keys(int flags, llist_iter func, void *c, error *err)
+int do_all_keys(kd_txn tid, int flags, llist_iter func, void *c, error *err)
 {
    akte_state aktes;
    int ret, i;
@@ -1067,7 +1160,7 @@ int do_all_keys(int flags, llist_iter func, void *c, error *err)
    ftfs.state = c;
 
    for (i=0; i<num_keydb; i++) {
-      if ((ret = (*(keydb_files[i]->cursor))(keydb_files[i], NULL, &cursor,
+      if ((ret = (*(keydb_files[i]->cursor))(keydb_files[i], tid, &cursor,
 					     0))) {
 	 err->fatal = 1;
 	 sprintf(err->buf, "error creating keydb[%d] cursor: error = %d",
@@ -1083,12 +1176,12 @@ int do_all_keys(int flags, llist_iter func, void *c, error *err)
 	       don't bother collecting id's or sorting */
 	    if (kd_keyblock_iterate((unsigned char *) data.data,
 				    (long) data.size,
-				    func_then_free, &ftfs, err, NULL))
+				    func_then_free, &ftfs, err, NULL, 0))
 	       continue;
 	 } else {
 	    if (kd_keyblock_iterate((unsigned char *) data.data,
 				    (long) data.size,
-				    add_key_to_entrylist, &aktes, err, NULL))
+				    add_key_to_entrylist, &aktes, err, NULL,0))
 	       continue;
 	 }
 
@@ -1138,6 +1231,7 @@ int do_all_keys(int flags, llist_iter func, void *c, error *err)
       qsort(aktes.entries.buf, (size_t) (aktes.entries.len/12), 12, 
 	    sort_twelvebytes);
 
+      ows.tid = tid;
       ows.userid = NULL;
       ows.userid_len = 0;
       ows.filter = NULL;
@@ -1166,8 +1260,8 @@ int do_all_keys(int flags, llist_iter func, void *c, error *err)
 
    */
 
-int do_by_userid(unsigned char *userid, long len, int flags, int maxkeys,
-		 llist_iter func, search_llist_filter filt,
+int do_by_userid(kd_txn tid, unsigned char *userid, long len, int flags,
+		 int maxkeys, llist_iter func, search_llist_filter filt,
 		 void *c, error *err)
 {
    llist words;
@@ -1177,6 +1271,13 @@ int do_by_userid(unsigned char *userid, long len, int flags, int maxkeys,
 
    llist_alloc(&words);
 
+   /* if the search userid is longer than any valid userid, fail. */
+   if (len > 255) {
+       err->fatal = 0;
+       err->str = "search userid may not be longer than 255 characters";
+       return(0);
+   }
+
    /* generate the word list */
 
    if (!kd_add_userid_to_wordlist(&words, userid, len)) {
@@ -1193,6 +1294,7 @@ int do_by_userid(unsigned char *userid, long len, int flags, int maxkeys,
       return(0);
    }
 
+   wkis.tid = tid;
    wkis.first = 1;
    llist_alloc(&(wkis.wdes));
    wkis.err = err;
@@ -1243,6 +1345,7 @@ int do_by_userid(unsigned char *userid, long len, int flags, int maxkeys,
    err->fatal = 1;
    strcpy(err->buf, "internal error generating key list");
 
+   ows.tid = tid;
    if (flags & KD_SEARCH_EXACT) {
       ows.userid = userid;
       ows.userid_len = len;
@@ -1278,13 +1381,28 @@ int parse_keyidstr(unsigned char *keyidstr, long len,
 {
    int i, digit;
 
-   if (len != KEYDB_KEYID_BYTES*2) {
+   /* Note the 32-character v3 fingerprint is not included here as it
+      is not possible to calculate a long or short keyid from it. -ds */
+
+   if (len != KEYDB_KEYID_BYTES*2 &&
+       len != KEYDB_KEYID_BYTES*4 &&
+       len != 40) {
       err->fatal = 0;
-      sprintf(err->buf, "Key ID string should be %d characters long",
-	      KEYDB_KEYID_BYTES*2);
+      sprintf(err->buf,"Key ID string should be %d, %d, or 40 characters long",
+	      KEYDB_KEYID_BYTES*2,KEYDB_KEYID_BYTES*4);
       return(0);
    }
 
+   /* If it is too big, shrink to the short 8-digit keyid.  This will
+      give a graceful transition to a future version that can handle
+      16-digit or full fingerprint keyids. -ds */
+
+   while(len>KEYDB_KEYID_BYTES*2)
+     {
+       keyidstr++;
+       len--;
+     }
+
    for (i=0; i<8; i++)
       keyid[i] = 0;
 
@@ -1311,12 +1429,12 @@ int parse_keyidstr(unsigned char *keyidstr, long len,
    return(1);
 }
 
-int kd_search_1(unsigned char *userid, long len, int flags, int maxkeys,
-		llist_iter func, search_llist_filter filt,
+int kd_search_1(kd_txn tid, unsigned char *userid, long len, int flags,
+		int maxkeys, llist_iter func, search_llist_filter filt,
 		void *c, error *err)
 {
    if (flags & KD_SEARCH_ALL)
-      return(do_all_keys(flags, func, c, err));
+      return(do_all_keys(tid, flags, func, c, err));
 
    /* skip over initial whitespace */
 
@@ -1348,6 +1466,7 @@ int kd_search_1(unsigned char *userid, long len, int flags, int maxkeys,
       err->fatal = 1;
       strcpy(err->buf, "internal error generating key list");
 
+      ows.tid = tid;
       ows.userid = NULL;
       ows.userid_len = 0;
       ows.filter = filt;
@@ -1361,7 +1480,7 @@ int kd_search_1(unsigned char *userid, long len, int flags, int maxkeys,
 
       return(1);
    } else {
-      if (!do_by_userid(userid, len, flags, maxkeys, func, filt, c, err))
+      if (!do_by_userid(tid, userid, len, flags, maxkeys, func, filt, c, err))
 	 return(0);
    }
 
