@@ -2,7 +2,7 @@
 #define _PGPUTIL_H_
 
 /*
- * $Id: pgputil.h,v 1.6 1999/05/26 03:35:28 marc Exp $
+ * $Id: pgputil.h,v 1.2 2003/01/26 16:08:58 dmshaw Exp $
  * 
  * Copyright (c) 1996, 1997, 1998, 1999, Marc Horowitz.  All rights reserved.
  * See the LICENSE file in the release for redistribution information.
@@ -29,7 +29,8 @@ typedef int (*packet_handler)(ddesc *packet, void *closure);
 int decode_num(ddesc *data, long size, long *num);
 int decode_psf(ddesc *data, long *ptype, long *plen);
 int decode_pubkey(ddesc *data, long len, mpidesc *modulus, mpidesc *exponent,
-		  unsigned char *keyid, int *keytype, time_t *create_time);
+		  unsigned char *keyid, unsigned char *keytype,
+		  time_t *create_time, int *keyversion);
 int decode_userid(ddesc *data, long len, ddesc *userid);
 int decode_sig(ddesc *data, long len, ddesc *keyid, long *sigclass,
 	       time_t *sigtime);
