@@ -2,7 +2,7 @@
 #define _PKS_CONFIG_H_
 
 /*
- * $Id: pks_config.h,v 1.11 1999/05/20 03:39:52 marc Exp $
+ * $Id: pks_config.h,v 1.5 2002/11/05 04:07:38 rlaager Exp $
  * 
  * Copyright (c) 1996, 1997, 1998, 1999, Marc Horowitz.  All rights reserved.
  * See the LICENSE file in the release for redistribution information.
@@ -15,7 +15,10 @@ typedef struct _pks_config {
    char db_dir[1024];
 
    /* web server stuff */
+   char www_dir[1024];
    int www_port;
+   char www_addr[1024];
+   int www_readonly;
 
    /* mail server stuff */
    char socket_name[1024];
@@ -32,6 +35,7 @@ typedef struct _pks_config {
 
    /* policy stuff */
    int max_last;
+   int max_last_reply_keys;
    int max_reply_keys;
 } pks_config;
 
