@@ -2,7 +2,7 @@
 #define _PKS_WWW_H_
 
 /*
- * $Id: pks_www.h,v 1.4 1999/05/20 03:39:52 marc Exp $
+ * $Id: pks_www.h,v 1.5 2002/11/05 04:07:38 rlaager Exp $
  * 
  * Copyright (c) 1996, Marc Horowitz.  All rights reserved.
  * See the LICENSE file in the release for redistribution information.
@@ -11,8 +11,11 @@
 #include "pks_incr.h"
 
 typedef struct _pks_www_conf {
+   char *addr;
    int port;
+   int readonly;
    int max_reply_keys;
+   char *www_dir;
    pks_incr_conf *pic;
 } pks_www_conf;
 
