@@ -1,4 +1,4 @@
-const char rcsid_wwwtest_c[] = "$Id: wwwtest.c,v 1.7 1996/12/11 19:18:04 marc Exp $";
+const char rcsid_wwwtest_c[] = "$Id: wwwtest.c,v 1.2 2003/02/07 01:01:21 rlaager Exp $";
 
 /* 
  * Copyright (c) 1996, Marc Horowitz.  All rights reserved.
@@ -7,6 +7,7 @@ const char rcsid_wwwtest_c[] = "$Id: wwwtest.c,v 1.7 1996/12/11 19:18:04 marc Ex
 
 
 #include <stdio.h>
+#include <stdlib.h>
 
 #include "www.h"
 #include "util.h"
@@ -37,7 +38,7 @@ void wwwtest(int fd, int vers, unsigned char *uri, long urilen,
       log_fatal("wwwtest", "appending to www reply");
    }
 
-   www_reply(fd, vers, 200, "OK", xb.buf, xb.len);
+   www_reply(fd, vers, 200, "OK", "text/plain", xb.buf, xb.len);
 
    xbuffer_free(&xb);
 }
@@ -47,8 +48,8 @@ int main(int argc, char *argv[])
    if (!mp_init())
       log_fatal("main", "failed initializing multiplexor");
 
-   www_init(8888, wwwtest, NULL);
-   www_init(9999, wwwtest, NULL);
+   www_init("", 8888, wwwtest, NULL);
+   www_init("", 9999, wwwtest, NULL);
 
    printf("wwwtest server started\n");
 
